local L = AceLibrary("AceLocale-2.2"):new("FarmerFu")
local Crayon = AceLibrary("Crayon-2.0")
local Tablet = AceLibrary("Tablet-2.0")
local Bags = AceLibrary("SpecialEvents-Bags-2.0")

local counts = {}
local options = {
  handler = FarmerFu,
  type = "group",
  args = {
    add = {
      type = "text",
      order = 1.0,
      name = L["Add"],
      desc = L["Add item to the list"],
      get = false,
      set = "AddItem",
      usage = L["Shift left click an item in your bag"],
      guiHidden = true,
    },
    watch = {
      type = "text",
      order = 1.1,
      name = L["Watch"],
      desc = L["Watch item on the list"],
      validate = {},
      get = "GetWatchItem",
      set = "SetWatchItem", 
    },
    remove = {
      type = "group",
      order = 1.2,
      name = L["Remove"],
      desc = L["Remove item from the list"],
      args = {},
    },
    clear = {
      type = "execute",
      order = 1.3,
      name = L["Clear List"],
      desc = L["Clear all items from the list"],
      func = "ClearItems",
    },
  },
}

FarmerFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "AceHook-2.1", "AceDebug-2.0")

FarmerFu.version = "2.0."..string.sub("$Revision: 19641 $", 12, -3)
FarmerFu.date = string.sub("$Date: 2006-12-08 19:17:01 -0500 (Fri, 08 Dec 2006) $", 8, 17)
FarmerFu.hasIcon = "Interface\\Icons\\Spell_Nature_Polymorph"
FarmerFu.independentProfile = true
FarmerFu.tooltipHiddenWhenEmpty = true
FarmerFu.debugFrame = ChatFrame5
FarmerFu.OnMenuRequest = options

FarmerFu:RegisterDB("FarmerFuDB")
FarmerFu:RegisterDefaults("profile", {
  items = {},
  watch = nil,
})
FarmerFu:RegisterChatCommand({"/farmerfu", "/farmfu", "/ffu"}, options)

function FarmerFu:OnEnable()
  self:RegisterEvent("SpecialEvents_BagSlotUpdate", "Update")
  self:SecureHook("ContainerFrameItemButton_OnClick")

  self:GetAllCounts()
  self:UpdateWatchOptions()
  self:UpdateRemoveOptions()
end

function FarmerFu:OnDataUpdate()
  self:GetAllCounts()
  self:UpdateWatchOptions()
  self:UpdateRemoveOptions()
end

function FarmerFu:OnTextUpdate()
  local item = self.db.profile.watch
  if item and self:IsFarming(item) then
    self:SetText(Crayon:Green(item).." ("..counts[item]..")")
  else
    self:SetText(self:GetTitle())
  end
end

function FarmerFu:OnTooltipUpdate()
  local r, g, b = 1, 1, 0
  local r2, g2, b2 = 1, 1, 1;
  local category

  category = Tablet:AddCategory(
    "columns", 2,
    "child_textR", r,
    "child_textG", g,
    "child_textB", b,
    "child_text2R", r2,
    "child_text2G", g2,
    "child_text2B", b2
  )

  table.sort(self.db.profile.items, function(a, b) return a < b end)
  for _, item in ipairs(self.db.profile.items) do
    category:AddLine(
      "text", item,
      "text2", counts[item]
    )
  end
end

function FarmerFu:GetAllCounts()
  -- Init the counts table
  counts = {}
  for _, item in ipairs(self.db.profile.items) do
    counts[item] = 0
  end
 
  -- Populate the counts table from the bags
  for bag = 0, NUM_BAG_FRAMES do
    for slot = 1, GetContainerNumSlots(bag) do
      local item = GetContainerItemLink(bag, slot) or ""
      item = self:RemoveHyperlink(item)
      if item and counts[item] then
        local _, count = GetContainerItemInfo(bag, slot)
        counts[item] = counts[item] + count
      end
    end
  end
end

function FarmerFu:UpdateWatchOptions()
  -- Repopulate it with all the items in the counts table.
  options.args.watch.validate = {} --Compost:Acquire()
  table.insert(options.args.watch.validate, L["None"])
  table.sort(self.db.profile.items, function(a, b) return a < b end)
  for _, item in ipairs(self.db.profile.items) do
    table.insert(options.args.watch.validate, item)
  end
end

function FarmerFu:UpdateRemoveOptions()
  -- Clear out the remove args table
  for item in pairs(options.args.remove.args) do
    options.args.remove.args[item] = nil
  end

  -- Repopulate it with all the items in the counts table.  If the
  -- counts table is empty, then disable the option.
  if next(counts) then
    options.args.remove.disabled = false
    table.sort(self.db.profile.items, function(a, b) return a < b end)
    for _, item in ipairs(self.db.profile.items) do
      local itemname = item
      local itemkey = string.gsub(itemname, " ", "")
      options.args.remove.args[itemkey] = {}
      options.args.remove.args[itemkey].type = "execute"
      options.args.remove.args[itemkey].name = itemname
      options.args.remove.args[itemkey].desc = L["Remove"].." "..itemname
      options.args.remove.args[itemkey].func = function() return self:RemoveItem(itemname) end
    end
  else
    options.args.remove.disabled = true
  end
end

function FarmerFu:AddItem(item)
  item = self:RemoveHyperlink(item)
  if item and not self:IsFarming(item) then
    table.insert(self.db.profile.items, item)
    self:Update()
  end
end

function FarmerFu:GetWatchItem()
  if self.db.profile.watch == nil then
    return L["None"]
  end
  return self.db.profile.watch
end

function FarmerFu:SetWatchItem(item)
  if item == L["None"] then
    self.db.profile.watch = nil
  elseif self:IsFarming(item) then
    self.db.profile.watch = item
  end
  self:UpdateText()
end

function FarmerFu:RemoveItem(item)
  local found = false
  for i, farmeditem in ipairs(self.db.profile.items) do
    if item == farmeditem then
      table.remove(self.db.profile.items, i)
      found = true
      break
    end
  end
  if found then
    counts[item] = nil
    if item == self.db.profile.watch then
      self.db.profile.watch = nil
    end
    self:Update()
  end
end

function FarmerFu:ClearItems()
  self.db.profile.watch = nil
  self.db.profile.items = {}
  counts = {}
  self:Update()
end

function FarmerFu:ContainerFrameItemButton_OnClick(button, index)
  if IsShiftKeyDown() and IsControlKeyDown() then
    local item = GetContainerItemLink(this:GetParent():GetID(), this:GetID()) or ""
    self:AddItem(item)
  end
end

function FarmerFu:RemoveHyperlink(item)
  return gsub(item, ".*%[(.*)%].*", "%1")
end

function FarmerFu:IsFarming(item)
  for _, farmeditem in ipairs(self.db.profile.items) do
    if item == farmeditem then
      return true
    end
  end
  return false
end
